<?php

namespace app\wx_url\controller;

use think\facade\Db;
use think\facade\View;
use vitphp\admin\controller\BaseController;
use think\facade\Request;

/**
 * @title 小程序管理
 */
class Url extends BaseController
{
    /**
     * @title 小程序列表
     */
    public function index()
    {
        $pid = input('pid');
        $wxapp = Db::name('wx_url_list')->where(['pid' => $pid])->order('id', 'desc')->select()->all();
        $url = Request::domain() . '/wx_url/';
        View::assign([
            'wxapp' => $wxapp,
            'url'=>$url
        ]);
        return View::fetch();
    }
    /**
     * @title 绑定小程序域名
     */
    public function domain(){
        $id = input('id');
        $pid = input('pid');
        $wxapp = Db::name('wx_url_list')->where(['id' => $id, 'pid' => $pid])->find();
        //判断是否post提交
        if(request()->isPost()){
            $domain = input('domain');
            if(!$domain){
                Db::name('routing')->where('name', $wxapp['code'])->where('pid', $pid)->delete();
                Db::name('wx_url_list')->where(['id' => $id])->update([
                    'domain' => null
                ]);
                $this->success("保存成功！", ToUrl('wx_url/url/index'));
            }
            $routing = Db::name('routing')->where('name', $wxapp['code'])->where('pid', $pid)->find();
            if($routing){
                Db::name('routing')->where('id', $routing['id'])->update([
                    'domain' => $domain
                ]);
            }else{
                Db::name('routing')->insert([
                    'pid' => $pid,
                    'name' => $wxapp['code'],
                    'domain' => $domain,
                    'url' => 'wx_url/' . $wxapp['code']
                ]);
            }
            Db::name('wx_url_list')->where(['id' => $id])->update([
                'domain' => $domain
            ]);
            $this->success("保存成功！", ToUrl('wx_url/url/index'));
        }
        View::assign([
            'wxapp' => $wxapp
        ]);
        return View::fetch();
    }
    /**
     * @title 编辑、新增小程序
     */
    public function edit()
    {
        $id = input('id');
        $pid = input('pid');
        $wxapp = Db::name('wx_url_list')->where(['id' => $id, 'pid' => $pid])->find();
        if($wxapp){
            $wxapp['time'] = $wxapp['time'] ? date("Y-m-d H:i:s", $wxapp['time']) : date("Y-m-d H:i:s", strtotime('+30 day'));
        }else{
            $wxapp['time'] = date("Y-m-d H:i:s", strtotime('+30 day'));
        }

        //判断是否post提交
        if (request()->isPost()) {
            $param = [
                'name' => input('name'),
                'wxapp_appid' => input('wxapp_appid'),
                'wxapp_appsecret' => input('wxapp_appsecret'),
                'wxapp_path' => input('wxapp_path'),
                'is_expire' => input('is_expire'),
                'days' => input('days'),
                'time' => strtotime(input('time')),
                'pid' => $pid
            ];
            if($param['is_expire']!=3) unset($param['time']);
            if($param['is_expire']!=2) unset($param['days']);
            if($param['is_expire']==2){
                if(!$param['days']) $this->error('请填写正确的有效天数');
                if(($param['days']%1!=0 || $param['days']<=0)) $this->error('请填写正确的有效天数！');
                if($param['days'] >= 365) $this->error('有效天数最长为365天');
                $path = [
                    'is_expire' => true,
                    'expire_type' => 1,
                    'expire_interval' => $param['days']
                ];
            }
            if($param['is_expire']==3){
                if(!$param['time']) $this->error('请选择正确的有效时间');
                if($param['time'] <= time()) $this->error('有效期最少要大于当下');
                if($param['time'] - time() >= 86400 * 30) $this->error('有效期最长为30天');
                $path = [
                    'is_expire' => true,
                    'expire_type' => 0,
                    'expire_time' => $param['time']
                ];
            }

            // 生成链接
            $config = [
                'appsecret' => $param['wxapp_appsecret'],
                'appid' => $param['wxapp_appid'],
            ];
            $mini = new \WeMini\Qrcode($config);
            $pathArr = $param['wxapp_path'] ? explode('?', $param['wxapp_path']) : [];
            if($pathArr){
                $path['path'] = $pathArr[0];
                $path['query'] = $pathArr[1] ?? '';
            }
            $path = $path ?? [];

            try {
                $wxapp = app_http_request("https://api.weixin.qq.com/wxa/generate_urllink?access_token=" . $mini->getAccessToken(), $path, [], [], true);
            }catch (\Exception $e){
                $this->error('微信配置有误：'.$e->getCode() . '-' . $e->getMessage());
            }
            $url = json_str_to_array($wxapp);
            if ($url["errcode"]!='0'){
                $this->error('链接配置有误：'. $url['errcode'] . '--' . $url['errmsg']);
            }

            if ($id) {
                Db::name('wx_url_list')->where('id', $id)->update($param);
            } else {
                $param['code'] = $this->getUrlCode();
                Db::name("wx_url_list")->insertGetId($param);
            }

            $this->success("保存成功！", "wx_url/index/index?pid=" . $pid);
        }
        View::assign([
            'wxapp' => !empty($wxapp) ? $wxapp : [],
        ]);
        return View::fetch();
    }

    protected function getUrlCode()
    {
        $code = redom(6, 2);
        $res = Db::name('wx_url_list')->where('code', '=', $code)->field('code')->find();
        if ($res) {
            return $this->getUrlCode();
        }
        return $code;
    }

    /**
     * @title 删除小程序
     */
    public function del()
    {
        $pid = input('pid');
        $ids = $this->request->post('ids');
        $wxapp = Db::name('v_url_list')->where(['id' => $ids, 'pid' => $pid])->find();
        if(!$wxapp) $this->error('未知的链接');

        Db::startTrans();
        try {
            Db::name('v_url_list')->where('id', $ids)->delete();
            Db::name('routing')->where('name', $wxapp['code'])->where('pid', $pid)->delete();
            Db::commit();
        }catch (\Exception $e){
            Db::rollback();
            throw $e;
        }
        $this->success('删除成功');
    }
}